/**
 * StyleChanger Ext Core extensions
 * Ext.ux.StyleChanger
 * version 1.0.3
 *
 * last updated 2022/10/1
 * author https://twitter.com/sou1ka
 * copyright http://www.plasmasphere.net/
 * example http://www.plasmasphere.net/archives/ext/stylechanger.html
 */

Ext.ns('Ext.ux');
Ext.ux.StyleChanger = Ext.extend(Ext.util.Observable, {
	renderTo: null,
	cssList: {
		href: null,
		html: null
	},
	fieldLabel: 'CSS:',
	cookieExpire: 60*60*24*7*1000, // 1week
	defaultStyleTitle: '',
	targetMediaType: '', // regexp

	styles: [],
	stylesDom: [],
	nothing: '無し',
	isRendered: false, // private

	constructor: function(config) {
		config = config || {};
		Ext.apply(this, config);
		Ext.ux.StyleChanger.superclass.constructor.call(this, config);
		this.addEvents(
			'change'
		);
		if(Ext.ux.Cookie) {
			this.cookie = new Ext.ux.Cookie({
				expires: new Date(new Date().getTime()+Number(this.cookieExpire))
			});
		}
		this.getStyle();
		this.setStyleFirst();
		if(this.renderTo) {
			Ext.onReady(function() {
				this.el = Ext.get(this.renderTo);
				this.initRender();
				this.initSelect();
			},this);
		}
	},

	initRender: function(){
		if(this.isRendered) {
			return;
		}
		this.html = Ext.DomHelper.markup(this.getDom());
		if(this.el) {
			this.selector = Ext.DomHelper.append(this.el, this.html, true);
		}
		this.isRendered = true;
		if(this.selector) {
			this.selector.child('select').on('change', function(e){
				this.cssChange(Ext.get(e.target).getValue() || e.target[e.target.selectedIndex].innerHTML);
			},this);
		}
	},

	getStyle: function() {
		if(!this.styles.length) {
			var link = Ext.select("link");
			var stylesLink = link.elements;
			for (var i = 0; i < stylesLink.length; i++) {
				if(stylesLink[i].media.match(new RegExp(this.targetMediaType))) {
					if(stylesLink[i].rel == "stylesheet" || stylesLink[i].rel == "alternate stylesheet") {
						this.styles.push({
							tag: 'option',
							html: stylesLink[i].title || stylesLink[i].href,
							title: stylesLink[i].href
						});
						this.stylesDom.push(stylesLink[i]);
					}
				}
			}
		}
	},

	cssChange: function(title, first) {
		if(Ext.isEmpty(title) || typeof title == "undefined") {
			title = this.defaultStyleTitle;
		}
		for (var i = 0; i < this.stylesDom.length; i++) {
			if(title != this.nothing) {
				if(this.stylesDom[i].title == title) {
					this.stylesDom[i].disabled = true;
					this.stylesDom[i].disabled = false;
				} else {
					this.stylesDom[i].disabled = true;
				}
			} else {
				this.stylesDom[i].disabled = true;
			}
		}
		if(this.cookie) {
			this.cookie.set('stylechanger', title);
		}
		this.fireEvent('change', title);
	},

	setStyleFirst: function() {
		if(this.cookie) {
			this.cssChange(this.cookie.get('stylechanger', this.defaultStyleTitle), true);
		}
	},

	initSelect: function(title) {
		var title = title || this.cookie.get('stylechanger', this.defaultStyleTitle);
		if(this.selector) {
			var ops = this.selector.select('option');
			if(ops&&ops.elements) {
				for(var j in ops.elements) {
					if(ops.elements[j].innerHTML==title) {
						ops.elements[j].selected = true;
					}
				}
			}
		}
	},

	getDom: function() {
		if(this.nothing) {
			this.styles.push({
				tag: 'option',
				html: this.nothing,
				title: this.nothing
			});
		}
		var select = {
			tag: 'span',
			id: 'StyleChanger',
			cls: 'StyleChanger',
			cn: [{
				tag: 'label',
				html: this.fieldLabel
			},{
				tag: 'select',
				name: 'StyleChanger-selector',
				cn: this.styles
			}]
		};
		if(!Ext.isEmpty(this.cssList)) {
			select.cn.push({
				tag: 'small',
				cn: [{
					tag: 'a',
					href: this.cssList.href,
					html: this.cssList.html
				}]
			});
		}
		return select;
	}
});

// setting
new Ext.ux.StyleChanger({
	renderTo: 'stylechanger',
	defaultStyleTitle: 'plasmasphere',
	targetMediaType: 'screen',
	cssList: {
		href: '/css_list.html',
		html: "&nbsp;" + 'CSS List'
	}
});